#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#
#set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../../lib/*.bash; do source "$f"; done

## Help text
show_usage() {
echo -n "
    -e|--environment        The environment name to create (defaults to 'qa')
    -o|--overwrite          Deletes the context prior to creation
"
}

### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    -e|--environment)
    ENVIRONMENT_NAME="$2"
    shift
    shift
    ;;
    -o|--overwrite)
    OVERWRITE=true
    shift
    ;;
    # --foo) # non value argument
    # INSTALL_FOO=true
    # shift
    # ;;
    # --bar) # argument with value 
    # INSTALL_BAR=true
    # BAR_VAL=$2
    # shift
    # shift
    # ;;
    *) # unknown option
    echo "Unknonwn option: $1"
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ -z ${ENVIRONMENT_NAME} ]]; then
    ENVIRONMENT_NAME=qa
fi

if [[ -z ${OX_ADMINMASTER_USER} ]] || [[ -z ${OX_MAIL_HOST} ]]; then
    echo "WARNING: The default configuration seems to be missing, Trying to load the default ... "
    loadConfig
fi

CONTEXT_CONFIG=${SCRIPT_DIR}/${ENVIRONMENT_NAME}/context
USERS_CSV=${SCRIPT_DIR}/${ENVIRONMENT_NAME}/users.csv
if [[ ! -f $CONTEXT_CONFIG ]]; then
    echo "$CONTEXT_CONFIG does not exist, aborting..."
    exit 1
fi
if [[ ! -f $USERS_CSV ]]; then
    echo "$USERS_CSV does not exist, aborting..."
    exit 1
fi

# loading context config
exportProperties $CONTEXT_CONFIG


echo "Creating context with following config:"
if [[ "${OVERWRITE}" = true ]]; then
    echo "Warning: Context ${CONTEXT_ID} (${CONTEXT_NAME}) will be overwritten with the current configuration if it exists!"
fi

printenv | grep "^CONTEXT_"
echo "=== (CTRL+C to cancel) ==="
echo ""
sleep 5

if [[ "${OVERWRITE}" = true ]]; then
    /opt/open-xchange/sbin/deletecontext -c "$CONTEXT_ID" -A "${OX_ADMINMASTER_USER}" -P "${OX_ADMINMASTER_PASS}" 2>/dev/null
fi

/opt/open-xchange/sbin/createcontext -A "${OX_ADMINMASTER_USER}" -P "${OX_ADMINMASTER_PASS}" \
                                        --contextid "${CONTEXT_ID}" \
                                        --contextname "${CONTEXT_NAME}" \
                                        --username "${CONTEXT_ADMIN_USER}" \
                                        --password "${CONTEXT_ADMIN_PASS}" \
                                        --displayname "${CONTEXT_ADMIN_USER}" \
                                        --givenname "${CONTEXT_ADMIN_USER}" \
                                        --surname "${CONTEXT_ADMIN_USER}" \
                                        --email "${CONTEXT_ADMIN_MAIL}" \
                                        --addmapping "${CONTEXT_LOGIN_MAPPING}" \
                                        --quota "${CONTEXT_QUOTA}" \
                                        --access-combination-name "${CONTEXT_ACN}"

if [[ $? != 0 ]]; then
    echo "Context could not be created, aborting."
    exit 1
fi

OLDIFS=$IFS
IFS=','
while read username password surname givenname displayname email language imaplogin imapserver smtpserver
do
    if [[ "$username" == "#"* ]]; then 
        # skipping commented out line 
        continue
    fi

    echo "User config:"
    echo "----------------------------------"
    #username,password,surname,givenname,displayname,email,language,imaplogin,imapserver,smtpserver
    echo "username:         $(eval echo "$username")"
    echo "password:         $(eval echo "$password")"
    echo "displayname:      $(eval echo "$displayname")"
    echo "surname:          $(eval echo "$surname")"
    echo "givenname:        $(eval echo "$givenname")"
    echo "email:            $(eval echo "$email")"
    echo "language:         $(eval echo "$language")"
    echo "imaplogin:        $(eval echo "$imaplogin")"
    echo "imapserver:       $(eval echo "$imapserver")"
    echo "smtpserver:       $(eval echo "$smtpserver")"
    echo "===================================="
    echo ""

    /opt/open-xchange/sbin/createuser --contextid "${CONTEXT_ID}" -A "${CONTEXT_ADMIN_USER}" -P "${CONTEXT_ADMIN_PASS}" \
                                        --username $(eval echo "$username") \
                                        --password $(eval echo "$password") \
                                        --displayname $(eval echo "$displayname") \
                                        --surname $(eval echo "$surname") \
                                        --givenname $(eval echo "$givenname") \
                                        --email $(eval echo "$email") \
                                        --imaplogin $(eval echo "$imaplogin") \
                                        --imapserver $(eval echo "$imapserver") \
                                        --smtpserver $(eval echo "$smtpserver") \
                                        --language $(eval echo "$language") \

done < $USERS_CSV
IFS=$OLDIFS

# TODO:
#   - additional config like virtual attachments

/opt/open-xchange/sbin/changecontext --contextid "${CONTEXT_ID}" -A "${OX_ADMINMASTER_USER}" -P "${OX_ADMINMASTER_PASS}" \
                                        --capabilities-to-add "${CONTEXT_CAPABILITIES}" \
                                        --access-global-address-book-disabled "${CONTEXT_DISABLE_GAB}"

