#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
# Philipp Schumacher <philipp.schumacher@open-xchange.com>

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]

    Available options:
      -i | --ip <ip-address> Add IP address as SAN to certificate
"
}

### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    -i|--ip)
    IP_ADDRESS="$2"
    shift
    ;;
    *) # unknown option
    echo "Unknonwn option: $1"
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

CERTIFICATE_PATH=/opt/open-xchange/certs

mkdir -p ${CERTIFICATE_PATH}

# Generate Certificate Authority Private Key
openssl genrsa -out ${CERTIFICATE_PATH}/ca.key 4096
# Create Certificate Authority Certificate
openssl req -new -x509 -days 365 -subj "/C=DE/ST=NRW/L=Olpe/O=Open-Xchange/CN=Quality Assurance Root CA" -key ${CERTIFICATE_PATH}/ca.key -out ${CERTIFICATE_PATH}/ca.cert.pem
# Need DER for Android devices and PEM for signing
openssl x509 -inform PEM -outform DER -in ${CERTIFICATE_PATH}/ca.cert.pem -out ${CERTIFICATE_PATH}/ca.cert.crt

if [ -n "$IP_ADDRESS" ] ; then
	cat > ${CERTIFICATE_PATH}/frontend.cnf <<EOF
[ req ]
prompt = no
x509_extensions = v3_req
distinguished_name = dn

[ dn ]
C = DE
ST = NRW
L = Olpe
O = Open-Xchange
CN = ${HOST_NAME}

[ v3_req ]
subjectAltName = @alt_names

[ v3_ext ]
subjectAltName = @alt_names

[ alt_names ]
IP.1 = $IP_ADDRESS
EOF
	# Generate a server key and request for signing (CSR)
	openssl genrsa -out ${CERTIFICATE_PATH}/frontend.key 4096
	openssl req -new -config ${CERTIFICATE_PATH}/frontend.cnf -key ${CERTIFICATE_PATH}/frontend.key -out ${CERTIFICATE_PATH}/frontend.csr
	# Sign the certificate with our CA
	openssl x509 -req -extensions v3_ext -extfile ${CERTIFICATE_PATH}/frontend.cnf -days 365 -in ${CERTIFICATE_PATH}/frontend.csr -CA ${CERTIFICATE_PATH}/ca.cert.pem -CAkey ${CERTIFICATE_PATH}/ca.key -CAcreateserial -out ${CERTIFICATE_PATH}/frontend.crt
else
	# Generate a server key and request for signing (CSR)
	openssl genrsa -out ${CERTIFICATE_PATH}/frontend.key 4096
	openssl req -new -subj "/C=DE/ST=NRW/L=Olpe/O=Open-Xchange/CN=${HOST_NAME}" -key ${CERTIFICATE_PATH}/frontend.key -out ${CERTIFICATE_PATH}/frontend.csr
	# Sign the certificate with our CA
	openssl x509 -req -days 365 -in ${CERTIFICATE_PATH}/frontend.csr -CA ${CERTIFICATE_PATH}/ca.cert.pem -CAkey ${CERTIFICATE_PATH}/ca.key -CAcreateserial -out ${CERTIFICATE_PATH}/frontend.crt
fi
# Generate p12 keystores
openssl pkcs12 -export -in ${CERTIFICATE_PATH}/ca.cert.pem -inkey ${CERTIFICATE_PATH}/ca.key -name "OX App Suite QA" -out ${CERTIFICATE_PATH}/ca-keystore.p12 -password pass:secret
openssl pkcs12 -export -in ${CERTIFICATE_PATH}/frontend.crt -inkey ${CERTIFICATE_PATH}/frontend.key -name "${HOST_NAME}" -out /${CERTIFICATE_PATH}/frontend-keystore.p12 -password pass:secret

#restartService ${HTTPD}