
#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Daniel Pondruff <daniel.pondruff@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]
  
    --enable
    --disable
    --restart
    -k | --key <value>

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    FEATURE_ENABLE=true
    shift
    ;;
    --disable)
    FEATURE_ENABLE=false
    shift
    ;;
    --install)
    FEATURE_INSTALL=true
    shift
    ;;
    --restart)
    RESTART_OX=true
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters


##TODO: make antivirus available at centos/rhel
if [[ -z "${ANY_DEB}" ]]; then
  echo "WARNING: Antivirus is at the moment only available on Debian..."
  echo ""
  export FEATURE_ANTIVIRUS=false
  sleep 10
  exit 0;
fi

if [[ "${FEATURE_ANTIVIRUS}" = true  ]]; then
  if [[ "${FEATURE_INSTALL}" = true  ]]; then  
    # TODO split anti virus setup into client/server architecture if possible
    # Currently it gets installed on any mw related node

    echo "Adding OX AntiVirus packages ..."
    installPackagesRetry "open-xchange-antivirus"

    echo "Adding c-icap and clamav ..."    
    installPackagesRetry "c-icap libc-icap-mod-virus-scan clamav-daemon"
  fi

  if [[ "${FEATURE_ENABLE}" = true  ]]; then  
    echo "Enabling OX anti virus feature ..."
    setOXProperty com.openexchange.antivirus.enabled true /opt/open-xchange/etc/antivirus.properties
    setOXProperty com.openexchange.antivirus.server localhost /opt/open-xchange/etc/antivirus.properties

    echo "Setting configurations for c-icap and clamav ..."
    #sed -i 's/ServerName.*/ServerName admin/g' /etc/c-icap/c-icap.conf
    #sed -i 's/ServerAdmin.*/ServerAdmin admin/g' /etc/c-icap/c-icap.conf
    #echo "Include virus_scan.conf" >> /etc/c-icap/c-icap.conf
    #sed -i 's/#Include clamav_mod.conf.*/Include clamav_mod.conf/g' /etc/c-icap/virus_scan.conf
    sed -i 's/START=no/START=yes/g' /etc/default/c-icap
    cat > /etc/c-icap/c-icap.conf <<EOL
PidFile /var/run/c-icap/c-icap.pid
CommandsSocket /var/run/c-icap/c-icap.ctl
Timeout 300
MaxKeepAliveRequests 100
KeepAliveTimeout 600
StartServers 3
MaxServers 10
MinSpareThreads     10
MaxSpareThreads     20
ThreadsPerChild     10
MaxRequestsPerChild  0
Port 1344
User c-icap
Group c-icap
ServerAdmin admin
ServerName admin
TmpDir /tmp
MaxMemObject 131072
DebugLevel 1
Pipelining on
SupportBuggyClients off
ModulesDir /usr/lib/x86_64-linux-gnu/c_icap
ServicesDir /usr/lib/x86_64-linux-gnu/c_icap
TemplateDir /usr/share/c_icap/templates/
TemplateDefaultLanguage en
LoadMagicFile /etc/c-icap/c-icap.magic
RemoteProxyUsers off
RemoteProxyUserHeader X-Authenticated-User
RemoteProxyUserHeaderEncoded on
ServerLog /var/log/c-icap/server.log
AccessLog /var/log/c-icap/access.log
Service echo srv_echo.so
Service antivirus_module virus_scan.so
ServiceAlias srv_clamav virus_scan
ServiceAlias  avscan virus_scan?allow204=off&sizelimit=off&mode=simple
virus_scan.ScanFileTypes TEXT DATA EXECUTABLE ARCHIVE GIF JPEG MSOFFICE
virus_scan.SendPercentData 5
virus_scan.StartSendPercentDataAfter 2M
virus_scan.MaxObjectSize  5M
Module common clamav_mod.so
clamav_mod.MaxScanSize 100M
EOL
    restartService clamav-freshclam

    ## TODO: Find better way of waiting for clamav-freshclam service restart ...
    sleep 20
    restartService c-icap
    # TODO: maybe switch to waitPort function which uses timeout
    until $(c-icap-client -i 127.0.0.1 -p 1344 -s avscan)
    do
      echo "Wait for c-icap service"
      sleep 1
    done

    # Touching restart required flag 
    touch /opt/open-xchange/etc/000-ox-restart-required
  elif [[ "${FEATURE_ENABLE}" = false  ]]; then  
    echo "Disabling Antivirus ..."
    setOXProperty com.openexchange.antivirus.enabled false /opt/open-xchange/etc/antivirus.properties
  
    # Touching restart required flag 
    touch /opt/open-xchange/etc/000-ox-restart-required
  fi
else
  echo "Skipping OX AntiVirus ..."
  echo ""
  sleep 3
fi

if [[ "${RESTART_OX}" = true  ]]; then
  echo "### Restart forced by script option ... "
  restartService open-xchange
fi



