
#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
source ${SCRIPT_DIR}/../lib/common.bash
source ${SCRIPT_DIR}/../lib/setup.bash

checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

SKIP_DOCKER_SETUP=false
BUILD_FROM_SOURCE=false 
if [[ -z "$AS_PROXY_VERSION" ]] ; then
    echo "WARNING: AS_PROXY_VERSION not set, fallback to develop"
    AS_PROXY_VERSION="develop"
fi

# log facility which simply mirrors the terminal output into a spcific file
if [[ -z "$AP_INSTALLER_LOG" ]] ; then
    AP_INSTALLER_LOG="${INSTALLER_ROOT}/appsuite-proxy-setup.log"
fi
if [[ -f "${AP_INSTALLER_LOG}" ]] ; then
    # removing log file if already exists (for image id parser)
    echo "removing existing appsuite proxy setup log file"
    rm "$AP_INSTALLER_LOG"
fi

exec > >(tee -ia "${AP_INSTALLER_LOG}")
exec 2> >(tee -ia "${AP_INSTALLER_LOG}" >&2)


## Help text
show_usage() {
echo -n "
  This script installs docker and creates required images and config for using the appsuite-proxy instead of apache as balancer.

  $0 [OPTIONS]

    --skip-docker-setup
    --build-from-source
    --skip-apache-config
    

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    # --enable)
    # FEATURE_ENABLE=true
    # shift
    # ;;
    # --disable)
    # FEATURE_ENABLE=false
    # shift
    # ;;
    --skip-docker-setup)
    SKIP_DOCKER_SETUP=true
    shift
    ;;
    --skip-apache-config)
    SKIP_APACHE_CONFIG=true
    shift
    ;;
    --build-from-source)
    BUILD_FROM_SOURCE=true
    shift
    ;; 
    -k|--key)
    value="$2"
    shift
    shift
    ;;
    --restart)
    RESTART_OX=true
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

echo "This script is intended for interactive use only atm. "
echo "You'll have to authenticate against gitlab for docker-compose and git client. "

sleep 5

if [[ ! -d /opt/appsuite-proxy ]]; then
  mkdir -p /opt/appsuite-proxy
fi

cd /opt/appsuite-proxy

# Ensure foreceHTTPS is set to false - this overrides the config!
setOXProperty com.openexchange.forceHTTPS false /opt/open-xchange/etc/server.properties
restartService open-xchange

if [ "$SKIP_DOCKER_SETUP" != true ]; then
  echo "Docker setup enabled, trying to install docker ..."

  echo -n "
  ==== WARNING - Use this only on test environments ====
  This script will automatically install docker as provided by get.docker.com
  You may want to review that script as it gets downloaded and executed with root privileges ...
  ================== CTRL+C to cancel ==================
  
  "
  sleep 3
  
  wget https://get.docker.com -O ${INSTALLER_ROOT}/get-docker.sh
  
  bash ${INSTALLER_ROOT}/get-docker.sh
  
  echo -n "
  
  Installing docker-compose ...
  "
  wget "https://github.com/docker/compose/releases/download/1.27.4/docker-compose-$(uname -s)-$(uname -m)" -O /usr/local/bin/docker-compose
  chmod +x /usr/local/bin/docker-compose


else
  echo "Skipping docker setup ..."
  sleep 3

fi

set +e

if [ "$BUILD_FROM_SOURCE" == true ] ; then
  echo "Building from appsuite-proxy source (BRANCH: ${AS_PROXY_VERSION})"
  echo "Installing Java as build dependency if enabled ..."
  if [ "$INSTALL_JAVA" = true ] ; then
      ${INSTALLER_ROOT}/setup/install-java.sh
  fi

  echo "Cloning appsuite-proxy repo (you may need to authenticate!)"
  git clone -b ${AS_PROXY_VERSION} https://gitlab.open-xchange.com/middleware/appsuite-proxy.git /opt/appsuite-proxy/src

  pushd /opt/appsuite-proxy/src

  /opt/appsuite-proxy/src/gradlew buildImage --stacktrace
  export IMAGE_ID=$(cat ${AP_INSTALLER_LOG} | grep "Created image with ID" | grep -Eo "'[a-z0-9]*'")

  popd

  echo "Creating config and env file for appsuite-proxy docker-compose ..."
  cp -Rv ${INSTALLER_ROOT}/config/appsuite-proxy/ /opt/

  echo -n "
  INSTALLER_ROOT=${INSTALLER_ROOT}
  IMAGE_ID=${IMAGE_ID}
  " > /opt/appsuite-proxy/.env
else
  echo "Using provided image from gitlab ..."
  
  echo "Creating config and env file for appsuite-proxy docker-compose ..."
  cp -Rv ${INSTALLER_ROOT}/config/appsuite-proxy/ /opt/

  echo -n "
  INSTALLER_ROOT=${INSTALLER_ROOT}
  IMAGE_ID=gitlab.open-xchange.com:4567/middleware/appsuite-proxy:${AS_PROXY_VERSION}
  " > /opt/appsuite-proxy/.env

  docker login gitlab.open-xchange.com:4567
  docker-compose pull
fi



#TODO this only works if the docker container can reach this hostname (e.g. openstack or propper local config)
sed -i "s/replace-me.invalid/${HOST_NAME}/g" /opt/appsuite-proxy/etc/backends.yml

if [ "$SKIP_APACHE_CONFIG" != true ]; then
  if [[ true == "${ANY_DEB}" ]]; then
    sed -i "s/Listen 80/Listen 8080/" /etc/${HTTPD}/ports.conf
    sed -i "s/Listen 443/Listen 8443/" /etc/${HTTPD}/ports.conf
    sed -i "s/VirtualHost _default_:80/VirtualHost _default_:8080/" /etc/${HTTPD}/sites-enabled/ox-http.conf
    sed -i "s/VirtualHost _default_:443/VirtualHost _default_:8443/" /etc/${HTTPD}/sites-enabled/ox-https.conf
  elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
    sed -i "s/Listen 80/Listen 8080/" /etc/${HTTPD}/conf/httpd.conf
    sed -i "s/Listen 443/Listen 8443/" /etc/${HTTPD}/conf.d/ssl.conf
    sed -i "s/VirtualHost _default_:80/VirtualHost _default_:8080/" /etc/${HTTPD}/conf.d/ox-http.conf
    sed -i "s/VirtualHost _default_:443/VirtualHost _default_:8443/" /etc/${HTTPD}/conf.d/ox-https.conf
  elif [[ true == "${ANY_SLES}" ]]; then
    echo "WARNING: SLES not implemented"
  fi

  restartService ${HTTPD}
fi
docker-compose config

sleep 3
docker-compose up -d

echo "App Suite proxy should now listen on Port 80 ... have a lot of fun"
set -e