
#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <daniel.pondruff@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]
  
    --enable
    --disable
    --restart
    -k | --key <value>

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    FEATURE_ENABLE=true
    shift
    ;;
    --disable)
    FEATURE_ENABLE=false
    shift
    ;;
    --install)
    FEATURE_INSTALL=true
    shift
    ;;
    -k|--key)
    value="$2"
    shift
    shift
    ;;
    --restart)
    RESTART_OX=true
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ "${FEATURE_CLIENT_ONBOARDING}" = true  ]]; then
  if [[ "${FEATURE_INSTALL}" = true  ]]; then  
    echo "Adding OX client-onboarding packages ..."
    PACKAGES="open-xchange-client-onboarding"
    installPackagesRetry "${PACKAGES}"
  fi

  if [[ "${FEATURE_ENABLE}" = true  ]]; then  

    echo "Setting configuration..."

    setOXProperty com.openexchange.client.onboarding.enabled true /opt/open-xchange/etc/client-onboarding.properties
    setOXProperty com.openexchange.client.onboarding.syncapp.store.google.playstore "https://play.google.com/store/apps/details?id=com.openexchange.mobile.syncapp.enterprise" /opt/open-xchange/etc/client-onboarding-syncapp.properties
    setOXProperty com.openexchange.client.onboarding.mail.imap.host ${OX_MAIL_HOST} /opt/open-xchange/etc/client-onboarding-mail.properties
    setOXProperty com.openexchange.client.onboarding.mail.smtp.host ${OX_MAIL_HOST} /opt/open-xchange/etc/client-onboarding-mail.properties

    cat > /opt/open-xchange/etc/client-onboarding-scenarios.yml <<EOL
--- 
davmanual: 
  alternatives: []
  description_t10e: "To manually setup your Calendar and Address Book accounts, please use the following information."
  displayName_t10e: "Calendar + Address Book"
  enabled: true
  icon: fa-wrench
  providers: 
    - carddav
    - caldav
  type: manual
davsync: 
  alternatives: 
    - davmanual
  description_t10e: "Synchronize your Calendar and Address Book data with your device."
  displayName_t10e: "Calendar + Address Book"
  enabled: true
  icon: "fa-calendar, fa-users"
  providers: 
    - carddav
    - caldav
  type: plist
driveappinstall: 
  alternatives: []
  description_t10e: "Synchronize your files with our Drive App."
  displayName_t10e: "Drive App"
  enabled: true
  icon: fa-cloud
  providers: 
    - driveapp
  type: link
drivewindowsclientinstall: 
  alternatives: []
  description_t10e: "Synchronize your files with our Drive application."
  displayName_t10e: "Drive Application"
  enabled: true
  icon: fa-cloud
  providers: 
    - drivewindowsclient
  type: link
easmanual: 
  alternatives: []
  description_t10e: "To manually setup your Mail, Calendar and Address Book accounts, please use the following information."
  displayName_t10e: "Mail + Calendar + Address Book"
  enabled: true
  icon: "fa-envelope-o, fa-calendar, fa-users"
  providers: 
    - eas
  type: manual
eassync: 
  alternatives: 
    - easmanual
  description_t10e: "Synchronize your Mail, Calendar and Address Book data with your device."
  displayName_t10e: "Mail + Calendar + Address Book"
  enabled: true
  icon: "fa-envelope-o, fa-calendar, fa-users"
  providers: 
    - eas
  type: plist
mailappinstall: 
  alternatives: []
  description_t10e: "Synchronize your Mail with our Mail App."
  displayName_t10e: "Mail App"
  enabled: true
  icon: fa-envelope-o
  providers: 
    - mailapp
  type: link
mailmanual: 
  alternatives: []
  description_t10e: "To manually setup your Mail account, please use the following information"
  displayName_t10e: Mail
  enabled: true
  icon: fa-envelope-o
  providers: 
    - mail
  type: manual
mailsync: 
  alternatives: 
    - mailmanual
  description_t10e: "Synchronize your Mail with Apple Mail App."
  displayName_t10e: Mail
  enabled: true
  icon: fa-envelope-o
  providers: 
    - mail
  type: plist
syncappinstall: 
  alternatives: []
  description_t10e: "Synchronize your Calendar and Address Book data with your device."
  displayName_t10e: "Calendar + Address Book"
  enabled: true
  icon: "fa-calendar, fa-users"
  providers: 
    - syncapp
  type: link
  
EOL


    # TODO: check if restart file exists and actually restart it
    touch /opt/open-xchange/etc/000-ox-restart-required

  elif [[ "${FEATURE_ENABLE}" = false  ]]; then
    echo "Disable client-onboarding..."
    setOXProperty com.openexchange.client.onboarding.enabled false /opt/open-xchange/etc/client-onboarding.properties
    
    echo "Reloading ox config ..."
    /opt/open-xchange/sbin/reloadconfiguration
  fi
else
  echo "Skipping feature client onboarding ..."
  echo ""
  sleep 3
fi


if [[ "${RESTART_OX}" = true  ]]; then
  echo "### Restart forced by script option ... "
  restartService open-xchange
fi