#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

if [[ -z "${AS_DOCUMENTCONVERTER_VERSION}" ]]; then
  # sourcing the default configuration to use
  echo "ERROR: Missing versions environment variable AS_DOCUMENTCONVERTER_VERSION"
  exit 99
fi

## Help text
show_usage() {
echo -n "

  $0 [OPTIONS]
    --singlenode    This flag combines install, enable, server, client and the frontend flag in order to set things up on a singlenode setup

    --install       The install flag
    --enable        Enable documents in config and restart the backend 
    --disable       Disable (WIP, may not work)
    --server        Set with install flag in order to install dc
    --client        Only configure documentconverter client with the provided remote dc (defaults to localhost)
    --restart     

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    DOCCONV_ENABLE=true
    shift
    ;;
    --disable)
    DOCCONV_ENABLE=false
    shift
    ;;
    --server)
    DOCCONV_SERVER=true
    shift
    ;;
    --client)
    DOCCONV_CLIENT=true
    shift
    ;;
    --install)
    export FEATURE_DOCUMENTCONVERTER=true
    shift
    ;;  
    --restart)
    RESTART_OX=true
    shift
    ;;  
    --singlenode)
    export DOCCONV_ENABLE=true
    export DOCCONV_SERVER=true
    export DOCCONV_CLIENT=true
    export FEATURE_DOCUMENTCONVERTER=true
    RESTART_OX=true
    shift
    ;;
    # -k|--key)
    # value="$2"
    # shift
    # shift
    # ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [ "$FEATURE_DOCUMENTCONVERTER" = true ] ; then
  echo "Document converter setup prepared ..."
  echo "===   (CTRL+C to cancel)   ==="
  sleep 3

  # Check if executing as singlenode setup
  if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
    DOCCONV_CLIENT=true
    DOCCONV_SERVER=true
  fi

  if [[ "${DOCCONV_SERVER}" = true  ]]; then    
    writeDocumentconverterRepositories
    writeDocumentconverterApiRepository

    echo "Adding OX documentconverter server packages ..."
    PACKAGES="readerengine open-xchange-documentconverter-client open-xchange-documentconverter-server open-xchange-documentconverter-api "
    installPackagesRetry "${PACKAGES}"

  fi
  # client will be installed by documents, frontned does not require packages
fi 

if [[ "${DOCCONV_ENABLE}" = true  ]]; then

  restartService open-xchange-documentconverter-server
  sleep 5 
  
  INIT_STATE_FILE="/opt/open-xchange/etc/21-init-documentconverter-done"

  if [[ "${DOCCONV_SERVER}" = true ]]; then
      echo "Doing some dc-server config"
  fi

  if [[ "${DOCCONV_CLIENT}" = true ]]; then
      if [[ -z "${CONVERTER_REMOTE_URL}" ]]; then
          CONVERTER_REMOTE_URL="http://localhost:8008/documentconverterws"
          echo "WARNING: Converter URL not provided, using fallback: $CONVERTER_REMOTE_URL"
          # TODO: This will work on singlenode only. 
          # In order to get this working in cluster envs use the apache proxy balancer 
      fi
      setOXProperty com.openexchange.documentconverter.client.remoteDocumentConverterUrl "${CONVERTER_REMOTE_URL}" /opt/open-xchange/etc/documentconverter-client.properties
      echo "Doing some dc-client config magic"
  fi

  if [[ "${FRONTEND}" = true ]]; then
      echo "WARNING: Frontend balancer config for documentconverter is not yet implemented"
      
  fi

  echo "$(date)" >> ${INIT_STATE_FILE}




elif [[ "${DOCCONV_ENABLE}" = false  ]]; then
  echo ""
  # do something to disable it
fi

if [[ "${RESTART_OX}" = true  ]]; then
  restartService open-xchange
fi