#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#
set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

if [[ -z "${AS_OFFICE_VERSION}" ]]; then
  # sourcing the default configuration to use
  echo "Warning: Missing versions environment variable AS_OFFICE_VERSION"
  exit 99
fi

## Help text
show_usage() {
echo -n "
Install Open-Xchange Guard packages and optionally enable it in config.
You can either install each component seperatly (manual cluster setup) or all at once.
Please keep in mind that most of this flags are already provided by the configuration.

  $0 [OPTIONS]
    
    --singlenode    This flag combines install, enable, middleware, frontend and the dcs flag in order to set things up on a singlenode setup

    --install       The install flag
    --enable        Enable documents in config and restart the backend 
    --disable       Disable (WIP, may not work)
    --restart       Wether or not to restart the all services after the action
    --middleware     Install middleware(api) packages only
    --frontend      Install Frontend packages
    -dcs | --documents-collaboration
    
"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    export DOCUMENTS_ENABLE=true
    shift
    ;;
    --disable)
    export DOCUMENTS_ENABLE=false
    shift
    ;;
    --middleware)
    export DOCUMENTS_MIDDLEWARE=true
    shift
    ;;
    --frontend)
    export DOCUMENTS_FRONTEND=true
    shift
    ;;
    -dcs|--documents-collaboration)
    export DOCUMENTS_INSTALL_DCS=true
    shift
    ;;
    --guard)
    export FEATURE_GUARD=true
    shift
    ;;   
    --install)
    export FEATURE_DOCUMENTS=true
    shift
    ;;    
    --singlenode)
    echo "Configuring documents for singlenode setup (documents, dcs, document converter) ..."
    
    export DOCUMENTS_INSTALL_DCS=true
    export DOCUMENTS_FRONTEND=true
    export DOCUMENTS_MIDDLEWARE=true
    export DOCUMENTS_ENABLE=true
    FEATURE_DOCUMENTS=true
    INSTALL_TYPE=singlenode
    RESTART_OX=true
    shift
    ;;
    # -k|--key)
    # value="$2"
    # shift
    # shift
    # ;;
    --restart)
    RESTART_OX=true
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

# Install documents middleware and frontend related packages
if [ "$FEATURE_DOCUMENTS" = true ] ; then
  # Check if executing as singlenode setup
  if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
    DOCUMENTS_MIDDLEWARE=true
    DOCUMENTS_FRONTEND=true
    DOCUMENTS_INSTALL_DCS=true
  fi

  if [[ "${DOCUMENTS_MIDDLEWARE}" = true  ]]; then
    echo "Adding OX Documents middleware packages ..."
    PACKAGES="${PACKAGES} open-xchange-documents-backend open-xchange-documents-templates open-xchange-documents-ui-common open-xchange-documents-ui-editors open-xchange-documentconverter-api"
    writeOfficeRepository
    writeDocumentconverterApiRepository
  fi

  if [[ "${DOCUMENTS_FRONTEND}" = true  ]]; then
    echo "Adding OX Documents frontend packages ..."
    PACKAGES="${PACKAGES} open-xchange-documents-ui-static open-xchange-documents-help-*"
    writeOfficeWebRepository
  fi

  echo "Documents setup prepared"
  echo "===   (CTRL+C to cancel)   ==="
  sleep 3

  if [[ -n "${PACKAGES}" ]]; then
    installPackagesRetry "${PACKAGES}"
  fi

  # Updating UI themes if script exists
  if [[ -f /opt/open-xchange/appsuite/share/update-themes.sh  ]]; then
    /opt/open-xchange/appsuite/share/update-themes.sh --if-needed
  fi
fi 

# Install and enable Documents collaboration server, client comes below
if [[ "$DOCUMENTS_INSTALL_DCS" = true ]] && [[ "${INSTALL_TYPE}" == *"singlenode"* ]] ; then
  ${SCRIPT_DIR}/documents-collaboration.sh --install --server --client  
fi

if [[ "${DOCUMENTS_ENABLE}" = true  ]]; then
  if [[ "$DOCUMENTS_INSTALL_DCS" = true ]] && [[ "${INSTALL_TYPE}" == *"singlenode"* ]] ; then
    # ensure that dcs server will be enabled if on any singlenode type (otherwise documents won't work due to missing dcs-configdb)
    echo "Enabling dcs for singlenode environment ..."
    ${SCRIPT_DIR}/documents-collaboration.sh --enable --client --server --restart            
  fi    
  
  setOXProperty permissions "text,spreadsheet,presentation,presenter,document_preview" /opt/open-xchange/etc/permissions.properties

  if [ "${FEATURE_GUARD}" = true ]; then
    setOXProperty com.openexchange.capability.guard-docs true /opt/open-xchange/etc/documents.properties
  fi

elif [[ "${DOCUMENTS_ENABLE}" = false  ]]; then
  echo ""
  # do something to disable it
fi

if [[ "${RESTART_OX}" = true  ]]; then
  restartService open-xchange
fi