#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Daniel Pondruff <daniel.pondruffopen-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi


## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]
  
    --install
    -k | --key <value>

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --install)
    FEATURE_INSTALL=true
    shift
    ;;
    -k|--key)
    value="$2"
    shift
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

INIT_STATE_FILE="/etc/dovecot/dovecot-init-done"

if [[ "${FEATURE_INSTALL}" = true  ]]; then
    if [[ -z "${DC_VERSION}" ]]; then
        echo "Version not specified, set to latest"
        DC_VERSION=2.3-latest
    fi
    if [[ true == "${ANY_DEB}" ]]; then
        # Adding dovecot key
        apt-key adv --fetch-keys https://repo.dovecot.org/DOVECOT-REPO-GPG
        {
        echo "deb https://repo.dovecot.org/ce-${DC_VERSION}/debian/${DIST_VERSION} ${DIST_VERSION} main"
        } > /etc/apt/sources.list.d/dovecot.list
      PACKAGES="dovecot-core dovecot-imapd dovecot-lmtpd dovecot-managesieved dovecot-sieve dovecot-pop3d dovecot-ldap postfix rsyslog"
    elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
        {
        echo "[dovecot-2.3-latest]
name=Dovecot 2.3 CentOS ${DIST_VERSION} - x86_64
baseurl=http://repo.dovecot.org/ce-2.3-latest/centos/${DIST_VERSION}/RPMS/x86_64
gpgkey=https://repo.dovecot.org/DOVECOT-REPO-GPG
gpgcheck=1
enabled=1"
        } > /etc/yum.repos.d/dovecot.repo
        yum makecache
        yum update
        PACKAGES="dovecot dovecot-pigeonhole"
    fi
    installPackages "${PACKAGES}"
    
    if [ ! -f ${INIT_STATE_FILE} ]; then
      echo "Installing dovecot packages"
      echo "$0 $@"
      echo "Creating user and group: vmail"
      groupadd -f vmail
      useradd -g vmail -m -d /home/vmail -s /bin/false vmail || true
      #adduser dovecot vmail
      usermod -a -G vmail dovecot
      # Create direcotries
      mkdir -p /var/log/dovecot
      mkdir -p /var/lib/dovecot/db
      mkdir -p /var/lib/dovecot/virtual/all
      touch /var/lib/dovecot/db/shared-mailboxes.db
    fi

    echo "Copy Dovecot Configs"
    rm -rf /etc/dovecot/*
    cp -R ${INSTALLER_ROOT}/config/dovecot/etc/dovecot/* /etc/dovecot/
    restartService dovecot
fi 

echo "*
-INBOX/Trash
-INBOX/Trash/*
-INBOX/Spam
-INBOX/Spam/*
  all" > /var/lib/dovecot/virtual/all/dovecot-virtual

LDA_PATH="/usr/lib/dovecot/dovecot-lda"
echo "dovecot     unix     -     n     n     -     -     pipe" >> /etc/postfix/master.cf
echo "  flags=DRhu user=vmail:vmail argv=${LDA_PATH} -f \${sender} -d \${user}" >> /etc/postfix/master.cf
echo "$(date)" > ${INIT_STATE_FILE}

echo "Re-configure postfix main.cf"
postconf -e "myhostname = dovecot"
postconf -e "smtpd_use_tls = no"
postconf -e "myorigin = \$mydomain"
postconf -e "mydestination = "
postconf -e "relayhost = "\$mydomain
#postconf -e "smtpd_tls_cert_file=${SSL_DIR}/certs/${FQDN}.crt"
#postconf -e "smtpd_tls_key_file=${SSL_DIR}/private/${FQDN}.key"
postconf -e "dovecot_destination_recipient_limit = 1"
postconf -e "virtual_mailbox_domains = ${HOST_NAME}"
postconf -e "virtual_transport = dovecot"
postconf -e "myorigin = \$mydomain"
postconf -e "mynetworks = 127.0.0.0/8 [::ffff:127.0.0.0]/104 [::1]/128 ${ENV_SUBNET}"
# set home permissions every time ...
chown -R vmail:vmail /home/vmail
chown -R vmail:vmail /var/lib/dovecot
chown -R vmail:vmail /var/log/dovecot*
chown -R vmail:vmail /var/run/dovecot/stats-*

chmod 2770 /home/vmail
chmod 0770 /var/lib/dovecot
chmod -R 700 /var/lib/dovecot/virtual

if [[ true == "${ANY_DEB}" ]]; then
    systemctl enable dovecot.service
elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
    # may not work on centos < 7
    systemctl enable dovecot.service
elif [[ true == "${ANY_SLES}" ]]; then
  echo "WARNING: SLES not implemented"
fi

restartService postfix
restartService dovecot
