
#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
#
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Daniel Pondruff <daniel.pondruff@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

if [[ -z "${AS_IMAGECONVERTER_VERSION}" ]]; then
  # sourcing the default configuration to use
  echo "ERROR: Missing versions environment variable AS_IMAGECONVERTER_VERSION"
  exit 99
fi

## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]
    --singlenode    This flag combines install, enable, server, client and the frontend flag in order to set things up on a singlenode setup

    --enable
    --disable
    --install     If set server or client components will be installed
    --server      Setups the server components
    --client      Setups the client components
    --remote-converter    Specify the remote image converter server used for client setup
    -k | --key <value>

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    FEATURE_ENABLE=true
    shift
    ;;
    --disable)
    FEATURE_ENABLE=false
    shift
    ;;
    --install)
    FEATURE_INSTALL=true
    shift
    ;;  
    --server)
    IMG_SERVER=true
    shift
    ;;
    --client)
    IMG_CLIENT=true
    shift
    ;;
    --remote-converter)
    IMG_REMOTE_HOST="$2"
    shift
    shift
    ;;
    --singlenode)
    export FEATURE_ENABLE=true
    export FEATURE_INSTALL=true
    export IMG_SERVER=true
    export IMG_CLIENT=true
    export FEATURE_IMAGECONVERTER=true
    RESTART_OX=true
    shift
    ;;
    -k|--key)
    value="$2"
    shift
    shift
    ;;
    --restart)
    RESTART_OX=true
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
  echo "Install type is singlenode, forcing client/server flags for imageconverter ..."

  IMG_CLIENT=true
  IMG_SERVER=true
fi

if [[ "${FEATURE_IMAGECONVERTER}" = true  ]]; then
  if [[ "${IMG_SERVER}" = true  ]]; then 
    writeImageConverterRepository
    if [[ true == "${ANY_DEB}" ]] ; then
      PACKAGES="imagemagick"
    elif [[ true == "${ANY_RHEL}" ]] ; then
      PACKAGES="ImageMagick"
    fi
    echo "Adding imageconverter server packages ..."
    PACKAGES="${PACKAGES} open-xchange-imageconverter-server "
  fi
  if [[ "${IMG_CLIENT}" = true  ]]; then  
    echo "Adding imageconverter client packages ..."
    PACKAGES="${PACKAGES} open-xchange-imageconverter-client"
  fi
  
  installPackagesRetry "${PACKAGES}"

  

  if [[ "${FEATURE_ENABLE}" = true  ]]; then  
    echo "Enabling FEATURE_IMAGECONVERTER ..."

    if [[ "${IMG_SERVER}" = true  ]]; then  
      echo "Setting up imageconverter-server ... "

      mkdir -p /var/opt/fileitemstore
      chown -R open-xchange:open-xchange /var/opt/fileitemstore
      waitTillAppsuiteReady

      if [ ! -z ${FEATURE_JAVA_SECURITY_MANAGER} ] || [ ${FEATURE_JAVA_SECURITY_MANAGER} ];then
        echo "if JSM is active, some things have to do here"
        if ! grep -q "Allow fileitemstore" /opt/open-xchange/etc/security/policies.policy;then
          echo "Setting FilePermission for Java Security manager..."
        sed -i '/\"name of permission\"/a\
  \nALLOW { \
    ( java.io.FilePermission "/var/opt/fileitemstore" "READ,WRITE" ) \
} "Allow fileitemstore"' /opt/open-xchange/etc/security/policies.policy
          restartService open-xchange
          waitTillAppsuiteReady
        else
          echo "Setting FilePermission for Java Security manager is already present"
        fi
      fi
      
      restartService open-xchange
      waitTillAppsuiteReady
      
      # Prevent fileitemstore from being used as default middleware filestore by providing '-x 0' flag
      /opt/open-xchange/sbin/registerfilestore -t file:///var/opt/fileitemstore \
            -A "${OX_ADMINMASTER_USER}" \
            -P "${OX_ADMINMASTER_PASS}" \
            -x 0 > /opt/open-xchange/etc/99-registerfileitemstore.out
      export OX_FILEITEMSTORE_ID=$(cat /opt/open-xchange/etc/99-registerfileitemstore.out | awk '{ print $2 }')
      echo "Fileitemstore ID is ${OX_FILEITEMSTORE_ID}"

      # Create a new, empty database schema to be used by the FileItemService:
      /opt/open-xchange/imageconverter/sbin/initfileitemdb -i \
            --fileitemdb-host="${DATABASE_HOST_NAME}" \
            --fileitemdb-user="${DATABASE_CONFIGDB_USER}" \
            --fileitemdb-pass="${DATABASE_CONFIGDB_PASS}" \
            --fileitemdb-port=${DATABASE_HOST_PORT} \
            --fileitemdb-dbname=fileitemdb

      #The following properties of the fileitem.properties need to be adjusted, at least (example values are provided):
      setOXProperty com.openexchange.fileItem.fileStoreIds ${OX_FILEITEMSTORE_ID} /opt/open-xchange/imageconverter/etc/fileitem.properties
      
      setOXProperty com.openexchange.fileItem.writeUrl "jdbc:mysql://${DATABASE_HOST_NAME}:${DATABASE_HOST_PORT}/fileitemdb" /opt/open-xchange/imageconverter/etc/fileitem.properties
      setOXProperty com.openexchange.fileItem.writeProperty.1 user=${DATABASE_CONFIGDB_USER} /opt/open-xchange/imageconverter/etc/fileitem.properties
      setOXProperty com.openexchange.fileItem.writeProperty.2 password=${DATABASE_CONFIGDB_PASS} /opt/open-xchange/imageconverter/etc/fileitem.properties
      
      setOXProperty com.openexchange.fileItem.readUrl "jdbc:mysql://${DATABASE_HOST_NAME}:${DATABASE_HOST_PORT}/fileitemdb" /opt/open-xchange/imageconverter/etc/fileitem.properties
      setOXProperty com.openexchange.fileItem.readProperty.1 user=${DATABASE_CONFIGDB_USER} /opt/open-xchange/imageconverter/etc/fileitem.properties
      setOXProperty com.openexchange.fileItem.readProperty.2 password=${DATABASE_CONFIGDB_PASS} /opt/open-xchange/imageconverter/etc/fileitem.properties
      
      restartService open-xchange-imageconverter-server

    fi

    if [[ "${IMG_CLIENT}" = true  ]]; then  
      if [[ -z "${IMG_REMOTE_HOST}" ]]; then
        echo "WARNING: No IMG_REMOTE_HOST was specified, using localhost as fallback ..."
        sleep 3
        IMG_REMOTE_HOST="http://localhost:8005/imageconverter"
      fi
  

      echo "Settings up open-xchange-imageconverter-client... "
      setOXProperty com.openexchange.imageconverter.client.remoteImageConverterUrl "${IMG_REMOTE_HOST}" /opt/open-xchange/etc/imageconverter-client.properties

      # Touching restart required flag 
      touch /opt/open-xchange/etc/000-ox-restart-required
    fi
    

  elif [[ "${FEATURE_ENABLE}" = false  ]]; then  
    echo "Disabling FEATURE_IMAGECONVERTER ..."
    if [[ "${IMG_SERVER}" = true  ]]; then  
      #echo "Disabling imageconverter server ..."
      echo ""

    fi
    if [[ "${IMG_CLIENT}" = true  ]]; then 
      echo "Disabling imageconverter client ... open-xchange restart may be required"
      setOXProperty com.openexchange.imageconverter.client.remoteImageConverterUrl "" /opt/open-xchange/etc/imageconverter-client.properties
    fi
    

  fi
else
  echo "Skipping feature imageconverter ..."
  echo ""
fi

if [[ "${RESTART_OX}" = true  ]]; then
  echo "### Restart forced by script option ... "
  restartService open-xchange-imageconverter-server
  restartService open-xchange
fi
