
#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
#
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Kai Ahrens <kai.ahrens@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

if [[ -z "${AS_SPELLCHECK_VERSION}" ]]; then
  # sourcing the default configuration to use
  echo "ERROR: Missing versions environment variable AS_SPELLCHECK_VERSION"
  exit 99
fi

## Help text
show_usage() {
echo -n "
  $0 [OPTIONS]
    --singlenode    This flag combines install, enable, server, client flag in order to set things up on a singlenode setup

    --enable
    --disable
    --install     If set server components will be installed
    --server      Setups the server components
    --remote-spellcheck    Specify the remote spellcheck server used for client setup
    -k | --key <value>

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    FEATURE_ENABLE=true
    shift
    ;;
    --disable)
    FEATURE_ENABLE=false
    shift
    ;;
    --install)
    FEATURE_INSTALL=true
    shift
    ;;
    --server)
    SPELLCHECK_SERVER=true
    shift
    ;;
    --remote-spellcheck)
    SPELLCHECK_REMOTE_HOST="$2"
    shift
    shift
    ;;
    --singlenode)
    FEATURE_INSTALL=true
    FEATURE_SPELLCHECK=true
    SPELLCHECK_SERVER=true
    FEATURE_ENABLE=true
    RESTART_OX=true
    shift
    ;;
    -k|--key)
    value="$2"
    shift
    shift
    ;;
    --restart)
    RESTART_OX=true
    shift
    ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
  echo "Install type is singlenode, forcing server flags for spellcheck ..."

  SPELLCHECK_SERVER=true
fi

if [[ "${FEATURE_SPELLCHECK}" = true  ]]; then
  if [[ "${SPELLCHECK_SERVER}" = true  ]]; then
    writeSpellcheckRepository
    echo "Adding spellcheck server packages ..."
    PACKAGES="open-xchange-spellcheck hunspell"
  fi

  installPackagesRetry "${PACKAGES}"

  if [[ "${FEATURE_ENABLE}" = true  ]]; then
    echo "Enabling FEATURE_SPELLCHECK ..."

    if [[ "${SPELLCHECK_SERVER}" = true  ]]; then
      echo "Setting up spellcheck service ... "

      restartService open-xchange-spellcheck
    fi

    if [[ -z "${SPELLCHECK_REMOTE_HOST}" ]]; then
      echo "WARNING: No SPELLCHECK_REMOTE_HOST was specified, using localhost as fallback ..."
      sleep 3
      SPELLCHECK_REMOTE_HOST="http://localhost:8003"
    fi


    echo "Setting up spellcheck client... "
    setOXProperty com.openexchange.office.spellcheck.remoteURL "${SPELLCHECK_REMOTE_HOST}" /opt/open-xchange/etc/documents.properties

    # Touching restart required flag
    touch /opt/open-xchange/etc/000-ox-restart-required

    if [ ! -z ${FEATURE_JAVA_SECURITY_MANAGER} ] || [ ${FEATURE_JAVA_SECURITY_MANAGER} ];then
      echo "if JSM is active, some things have to do here"
      if ! grep -q "Allow fileitemstore" /opt/open-xchange/etc/security/policies.policy;then
        echo "Setting FilePermission for Java Security manager..."
      sed -i '/\"name of permission\"/a\
  \nALLOW { \
      ( java.io.FilePermission "/var/opt/fileitemstore" "READ,WRITE" ) \
  } "Allow fileitemstore"' /opt/open-xchange/etc/security/policies.policy
        # Touching restart required flag - If not yet touched, it gets touched now - restart follow at 90_init_check_restart.sh
        touch /opt/open-xchange/etc/000-ox-restart-required
      else
        echo "Setting FilePermission for Java Security manager is already present"
      fi
    fi

  elif [[ "${FEATURE_ENABLE}" = false  ]]; then
    echo "Disabling FEATURE_SPELLCHECK ..."
    if [[ "${SPELLCHECK_SERVER}" = true  ]]; then
      #echo "Disabling spellcheck server ..."
      echo ""

    fi
    echo "Disabling spellcheck client ..."
    setOXProperty com.openexchange.office.spellcheck.remoteURL "" /opt/open-xchange/etc/documents.properties
    echo "Reloading ox config ..."
    /opt/open-xchange/sbin/reloadconfiguration
  fi
else
  echo "Skipping feature imageconverter ..."
  echo ""
fi

if [[ "${RESTART_OX}" = true  ]]; then
  restartService open-xchange-spellcheck
  restartService open-xchange
fi
