
#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi


## Help text
show_usage() {
echo -n "

  $0 [OPTIONS]
    
    --singlenode    This flag combines install, enable, middleware, frontend and the guard admin flag in order to set things up on a singlenode setup

    --install       The install flag
    --enable        Enable guard in config and restart the backend several times
    --middleware    Install middleware(api) packages only
    --frontend      Install Frontend balancer rules
    
"
}



### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --enable)
    FEATURE_ENABLE=true
    shift
    ;;
    --disable)
    FEATURE_ENABLE=false
    shift
    ;;
    --middleware)
    export USMEAS_MIDDLEWARE=true
    shift
    ;;
    --frontend)
    export USMEAS_FRONTEND=true
    shift
    ;;  
    --install)
    FEATURE_USMEAS=true
    shift
    ;;    
    --singlenode)
    # this option is probably redundant as this scripts checks for singlenode setup it self
    echo "Configuring usm-eas for singlenode setup ..."
    FEATURE_USMEAS=true
    export USMEAS_MIDDLEWARE=true
    export USMEAS_FRONTEND=true
    INSTALL_TYPE=singlenode
    RESTART_OX=true
    shift
    ;;
    # -k|--key)
    # value="$2"
    # shift
    # shift
    # ;;
    # --restart)
    # RESTART_OX=true
    # shift
    # ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters


if [[ "${FEATURE_USMEAS}" = true  ]]; then

  if [[ -z "${AS_EAS_VERSION}" ]]; then
    # sourcing the default configuration to use
    echo "ERROR: Missing versions environment variable AS_EAS_VERSION"
    exit 99
  fi
  # Check if executing as singlenode setup
  if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
    # Installing all required components for a singlenode environment
    echo "Executing usm eas feature script on singlenode environment ..."
    USMEAS_FRONTEND=true
    USMEAS_MIDDLEWARE=true
  else
    echo "USMEAS_FRONTEND and USMEAS_MIDDLEWARE flag are probably set by environment variables "
    # Due to this we will skip setting flags manually here and the environment will provide everything as requried (or the admin)
    echo "Executing usm eas feature script on unspecified environment ... You may take care of mw/fw on your own ..."
  fi

  if [[ "${USMEAS_MIDDLEWARE}" = true  ]]; then
    writeUsmEasRepositories
    USMEAS_PACKAGES="open-xchange-usm open-xchange-eas"
    installPackagesRetry "${USMEAS_PACKAGES}"
  fi
  if [[ "${USMEAS_FRONTEND}" = true  ]]; then
    echo "Adding usmeas proxy rules ..." 
    cp -f ${SCRIPT_DIR}/../config/frontend/conf/85_usmeas-proxy.dummy ${HTTPD_CONF_PATH}/85_usmeas-proxy.conf
  fi
  

  if [[ "${FEATURE_ENABLE}" = true  ]]; then  

    if [[ "${USMEAS_FRONTEND}" = true  ]]; then
      if [[ "${INSTALL_TYPE}" == *"singlenode"* ]] || [[ "${INSTALL_FRONTEND}" = true  ]]; then
        echo "Adding frontend balancer for eas-usm feature ..."
        ${SCRIPT_DIR}/../config/frontend/add-balancer.sh \
            --name "oxcluster_eas" \
            --balancer "http://localhost:8009" \
            --route "singlenode" \
            --priority 80

      fi
    fi
    # Touching restart required flag 
    touch /opt/open-xchange/etc/000-ox-restart-required
  elif [[ "${FEATURE_ENABLE}" = false  ]]; then  
    echo "Disabling FEATURE_USMEAS ..."
    setOXProperty feature "$value" /opt/open-xchange/etc/feature.properties

    # Touching restart required flag 
    touch /opt/open-xchange/etc/000-ox-restart-required
  fi
else
  echo "Skipping FEATURE_USMEAS ..."
  echo ""
  sleep 3
fi

if [[ "${RESTART_OX}" = true  ]]; then
  echo "### Restart forced by script option ... "
  restartService open-xchange
fi
