#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Philipp Schumacher <philipp.schumacher@open-xchange.com>
#

set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done
checkRootPrivileges
detectDistribution

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

if [[ -z "${AS_WEAKFORCE_VERSION}" && -z "${WF_VERSION}" ]]; then
  # sourcing the default configuration to use
  echo "ERROR: Missing one of the versions environment variables AS_WEAKFORCE_VERSION or WF_VERSION"
  exit 99
fi

if [[ true == "${ANY_RHEL}" ]] ; then
  echo "WARNING: Weakforce is currently not available for RHEL, skipping it for now ..."
  sleep 5
  exit 0
fi

## Help text
show_usage() {
echo -n "

  $0 [OPTIONS]
    --singlenode    This flag combines install, server and client in order to set things up on a singlenode setup

    --install       The install flag
    --server        Set with install flag in order to install wforce server only
    --client        Set with install flag in order to install weakforce backend plugin only. The follwoing environment variable can be used for configuration purposes
                        WFORCE_REMOTE_URL (default: 'http://localhost:8084')
                        WFORCE_HASH_SECRET (default: 'secretHash')
                        WFORCE_BASIC_AUTH_PASSWORD (default: 'secret')

"
}


### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --server)
    export WFORCE_SERVER=true
    shift
    ;;
    --backend-plugin)
    export WFORCE_BACKEND_PLUGIN=true
    shift
    ;;
    --install)
    export FEATURE_WEAKFORCE=true
    shift
    ;;  
    --restart)
    RESTART_OX=true
    shift
    ;;  
    --singlenode)
    export WFORCE_ENABLE=true
    export WFORCE_SERVER=true
    export WFORCE_BACKEND_PLUGIN=true
    export FEATURE_WEAKFORCE=true
    shift
    ;;
    # -k|--key)
    # value="$2"
    # shift
    # shift
    # ;;
    *)    # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters


if [ "$FEATURE_WEAKFORCE" = true ] ; then
  echo "Weakforce setup prepared ..."
  echo "===   (CTRL+C to cancel)   ==="
  sleep 3

  writeWeakforceRepository

  if [[ "${WFORCE_SERVER}" = true  ]]; then
    echo "Adding Weakforce server package ..."
    PACKAGES="wforce "
    installPackagesRetry "${PACKAGES}"
    echo "Doing some wforce server config magic"
    sed -i 's/webserver(\"0.0.0.0:8084\", \".*\")/webserver(\"0.0.0.0:8084\", \"secret\")/g' /etc/wforce/wforce.conf
    sed -i 's/setKey(\".*\")/setKey(\"secretHash\")/g' /etc/wforce/wforce.conf
    restartService wforce
  fi

  if [[ "${WFORCE_BACKEND_PLUGIN}" = true  ]]; then
    echo "Adding Weakforce Backend plugin package ..."
    PACKAGES="open-xchange-weakforced "
    installPackagesRetry "${PACKAGES}"
    echo "Doing some wforce backend plugin config magic"

    if [[ -z "${WFORCE_REMOTE_URL}" ]]; then
          WFORCE_REMOTE_URL="http://localhost:8084"
          echo "WARNING: Weakforce URL not provided, using fallback: $WFORCE_REMOTE_URL"
    fi

    if [[ -z "${WFORCE_HASH_SECRET}" ]]; then
        WFORCE_HASH_SECRET="secretHash"
        echo "WARNING: Weakforce hash secret not provided, using fallback: $WFORCE_HASH_SECRET"
    fi

    if [[ -z "${WFORCE_BASIC_AUTH_PASSWORD}" ]]; then
        WFORCE_BASIC_AUTH_PASSWORD="secret"
        echo "WARNING: Weakforce basic-auth password not provided, using fallback: $WFORCE_BASIC_AUTH_PASSWORD"
    fi

    setOXProperty com.openexchange.weakforced.endpoints "${WFORCE_REMOTE_URL}" /opt/open-xchange/etc/weakforced.properties
    setOXProperty com.openexchange.weakforced.hash.secret "${WFORCE_HASH_SECRET}" /opt/open-xchange/etc/weakforced.properties
    setOXProperty com.openexchange.weakforced.basic-auth.login "wforce" /opt/open-xchange/etc/weakforced.properties
    setOXProperty com.openexchange.weakforced.basic-auth.password "${WFORCE_BASIC_AUTH_PASSWORD}" /opt/open-xchange/etc/weakforced.properties
    restartService open-xchange
  fi

else
  echo "Skipping feature weakforce ..."
  echo ""
fi