#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
#
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#
set -e

SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"

pushd ${SCRIPT_DIR}
STARTED_AT=$(date)

# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/lib/*.bash; do source "$f"; done

loadConfig

# log facility which simply mirrors the terminal output into a spcific file
if [[ -z "$INSTALLER_LOG" ]] ; then
    INSTALLER_LOG=ox-installer.log
fi
exec > >(tee -ia ${INSTALLER_LOG})
exec 2> >(tee -ia ${INSTALLER_LOG} >&2)

checkRootPrivileges
detectDistribution

## Help text
show_usage() {
echo -n "

This scripts lets you install a fully fledged Open-Xchange App Suite
environment with configured database and dovecot backends.

Easiest way to get a working OX App Suite this:
    $ ./ox-installer.sh --singlenode

    Per component options (can be combined):
        --middleware        Installs middleware components
        --frontend          Installs frontnend with apache2 balancer
        --database          Installs mysql or mariadb server
        -d | --dovecot      Installs local only dovecot test server

    Configuration options:
        --init                     This will start configuration scripts after component installation (enabled by default in config)
        --config <file>            Default configuration file to load (default: config/config)
        --extended-config <file>   Extended options file to load (default: config/config.extended)

    Misc options:
        --update-same-version          This will update your environment on same version (for develop installations)
        --write-repos-only      Only write the available repositores and end the script (for debugging)

Examples:

Singlenode setup with everything you need:
    $ ./ox-installer.sh --dovecot

"
}

### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --middleware)
    export INSTALL_MIDDLEWARE=true
    shift
    ;;
    --frontend)
    export INSTALL_FRONTEND=true
    shift
    ;;
    --database)
    export INSTALL_DATABASE=true
    shift
    ;;
    --guard)
    export FEATURE_GUARD=true
    shift
    ;;
    --documents)
    export FEATURE_DOCUMENTS=true
    shift
    ;;
    --documentconverter)
    export FEATURE_DOCUMENTCONVERTER=true
    shift
    ;;
    --imageconverter)
    export FEATURE_IMAGECONVERTER=true
    shift
    ;;
    --spellcheck)
    export FEATURE_SPELLCHECK=true
    shift
    ;;
    --weakforce)
    export FEATURE_WEAKFORCE=true
    shift
    ;;
    -d|--dovecot)
    export INSTALL_DOVECOT=true
    export OX_MAIL_HOST=localhost
    export OX_MAIL_DOMAIN=${HOST_NAME}
    echo "WARNING: overriding OX_MAIL_HOST and OX_MAIL_DOMAIN with local only dovecot configuration!"
    sleep 3
    shift
    ;;
    --java)
    export INSTALL_JAVA=true
    shift
    ;;
    -s|--singlenode)
    export INSTALL_DATABASE=true
    export INSTALL_MIDDLEWARE=true
    export INSTALL_FRONTEND=true
    export INSTALL_TYPE=singlenode
    shift
    ;;
    --update-same-version)
    export UPDATE_SAME_VERSION=true
    features/update-same-version.sh
    exit 0
    shift
    ;;
    --config)
    export CONFIG_FILE="$2"
    exportProperties ${CONFIG_FILE}
    shift
    shift
    ;;
    --extended-config)
    export CONFIG_EXTENDED_FILE="$2"
    exportProperties ${CONFIG_EXTENDED_FILE}
    shift
    shift
    ;;
    --pre-install-actions)
    echo "Only executing pre install actions and leaving ..."
    preInstallActions
    echo "... Done "
    exit 0
    shift
    ;;
    --write-repos-only)
    echo ""
    echo "Forcefully writing repositories only!"
    sleep 5
    writeRepositories
    echo "... repo writing should be done"
    exit 0
    shift
    ;;
    *) # unknown option
    echo "Unknonwn option: $1"
    POSITIONAL+=("$1") # save it in an array for later
    shift
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

# checking if Install variables are set, if not default will be set to false
checkInstallDefaults

echo ""
echo "Setup checks done... Following environment is prepared:"
echo "===   (CTRL+C to cancel)   ==="
printenv | grep "^DATABASE_"
printenv | grep "^OX_"
echo "The following versions will be used:"
printenv | grep "^AS_"
printenv | grep "^WF_"
echo "The following components will be installed:"
printenv | grep "^INSTALL_"
echo "The following features will be installed:"
printenv | grep "^FEATURE_"
echo "===   (CTRL+C to cancel)   ==="
sleep 5

#writeRepositories
preInstallActions
addOxBuildKey

if [ "$INSTALL_JAVA" = true ] ; then
    setup/install-java.sh
fi

if [ "$INSTALL_DATABASE" = true ] ; then
    setup/install-database.sh
fi

if [ "$INSTALL_MIDDLEWARE" = true ] ; then
    # Install OX middleware (optionally with documents enabled)
    setup/install-middleware.sh
fi

if [ "$INSTALL_FRONTEND" = true ] ; then
    # Install OX App Suite frontend
    setup/install-frontend.sh
fi

if [ "$FEATURE_DOCUMENTS" = true ] ; then
    if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
        # ensure that all components will be "force" installed if on any singlenode
        echo "Installing documents for singlenode environment ..."
        features/documents.sh --install --middleware --frontend --documents-collaboration
    else
        # let the environment variables do the selection magic
        echo "Installing documents ..."
        features/documents.sh --install --middleware --documents-collaboration
    fi
fi

if [ "$FEATURE_DOCUMENTCONVERTER" = true ] ; then
    if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
        # ensure that all components will be "force" installed if on any singlenode
        echo "Installing documents for singlenode environment ..."
        features/documentconverter.sh --install --server --client
    else
        # let the environment variables do the selection magic
        echo "Installing documents ..."
        features/documentconverter.sh --install --server
    fi
fi

if [ "$FEATURE_GUARD" = true ] ; then
    if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
        # ensure that all components will be "force" installed if on any singlenode
        echo "Installing guard for singlenode environment ..."
        features/guard.sh --install --middleware --backend --frontend
    fi
fi

if [ "$FEATURE_WEAKFORCE" = true ] ; then
    if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
        # ensure that all components will be "force" installed if on any singlenode
        echo "Installing weakforce for singlenode environment ..."
        features/weakforce.sh --install --server --backend-plugin
    fi
fi

if [ "$INSTALL_DOVECOT" = true ] ; then
    if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then
        # ensure that all components will be "force" installed if on any singlenode
        echo "Installing documents for singlenode environment ..."
        features/dovecot.sh --install --enable
    fi
fi

if [[ "${INSTALL_TYPE}" == *"singlenode"* ]]; then

    features/usm-eas.sh --install
    features/client-onboarding.sh --install
    features/user-feedback.sh --install
    features/gdpr-data-export.sh --install
    features/antivirus.sh --install
    features/imageconverter.sh --install
    features/spellcheck.sh --install
    features/app-specific-password.sh --install
    features/drive-client-windows.sh --install

fi 

if [ "${OX_INIT}" = true ]; then
    echo "Init flag set to true... starting environment init ..."
    sleep 5

    if [ "$INSTALL_DATABASE" = true ] ; then
        config/init-database.sh
    fi

    if [ "$INSTALL_FRONTEND" = true ] ; then
        echo ""
        # Nothing todo for frontend init right now
        # It is done while middleware init scripts are running: config/middleware/30 ...

    fi
    if [ "$INSTALL_MIDDLEWARE" = true ] ; then
        config/init-middleware.sh
    fi

fi

FINISHED_AT=$(date)
echo "====================================================================================="
echo "Started at:   $STARTED_AT"
echo "Finished at:  $FINISHED_AT"

popd