#!/bin/bash
#
# Copyright (C) Open-Xchange GmbH, Germany <info@open-xchange.com>
# 
# This file is part of OX Automation.
#
# OX Automation is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# OX Automation is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OX Automation. If not, see <http://www.gnu.org/licenses/>.
#
#
# Authors:
# Benedikt Kroening <benedikt.kroening@open-xchange.com>
#
set -e
SCRIPT_DIR="$( cd "$(dirname "$0")" ; pwd -P)"
# sourcing some commonly used functions and executing basic checks
for f in ${SCRIPT_DIR}/../lib/*.bash; do source "$f"; done

DB_SERVICE_NAME=mysql

if [ "$CONFIG_LOADED" != true ] ; then
  loadConfig
fi

## Help text
show_usage() {
echo -n "
    --type              mysql|mariadb
    --version           5.6 ...
    --root-password
    --allow-root-from-all (mysql only)
    --init

"
}

### reading config from command line; overriding defaults
POSITIONAL=()
while [[ $# -gt 0 ]]
do
key="$1"

case $key in
    -h|--help)
    show_usage
    exit 0
    shift
    ;;
    --debug)
    echo "WARNING: Debug flag set (--debug)"
    set -x
    shift
    ;;
    --type)
    export DATABASE_TYPE=$2
    shift
    shift
    ;;
    --version)
    export DATABASE_VERSION=$2
    shift
    shift
    ;;
    --root-password)
    export DATABASE_ROOT_PASS="$2"
    shift
    shift
    ;;
    --init)
    export OX_INIT=true;
    shift
    ;;
    --allow-root-from-all)
    export DATABASE_ROOT_ALLOW_FROM_ALL=true
    shift
    ;;
    # --foo) # non value argument
    # INSTALL_FOO=true
    # shift
    # ;;
    # --bar) # argument with value 
    # INSTALL_BAR=true
    # BAR_VAL=$2
    # shift
    # shift
    # ;;
    *) # unknown option
    echo "Unknonwn option: $1"
    POSITIONAL+=("$1") # save it in an array for later
    shift 
    ;;
esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ -z ${DATABASE_TYPE} ]]; then
    export DATABASE_TYPE="mysql"
fi

if [[ -z ${DATABASE_VERSION} ]]; then
    export DATABASE_VERSION="5.7"
fi

checkRootPrivileges
detectDistribution

if [[ ${DATABASE_VERSION} == "5.6" ]] && [[ ${DIST} == "DebianBuster" ]]; then
    echo "ERROR: Debian Buster does not support mysql 5.6"
    exit 99
fi

echo "Database setup prepared, Following config will be used"
printenv | grep DATABASE_
echo "===   (CTRL+C to cancel)   ==="

sleep 3


preInstallActions

if [[ "${DATABASE_TYPE}" == "mysql" ]]; then
    echo "Installing mysql-server packages..."
    echo "Adding MySQL repositories"

    if [[ true == "${ANY_DEB}" ]]; then
        DB_SERVICE_NAME=mysql
        # Adding mysql key
        gpg --keyserver keyserver.ubuntu.com --recv-key 467B942D3A79BD29
        gpg --export -a 467B942D3A79BD29 > mysql-repo.key  
        apt-key add mysql-repo.key
        
        {
        echo "deb https://repo.mysql.com/apt/debian/ ${DIST_VERSION} mysql-$DATABASE_VERSION"
        echo "deb https://repo.mysql.com/apt/debian/ ${DIST_VERSION} mysql-tools"
        # echo "deb-src https://repo.mysql.com/apt/debian/ stretch mysql-$DATABASE_VERSION"
        } > /etc/apt/sources.list.d/mysql.list

    elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
        DB_SERVICE_NAME=mysqld
        gpg --keyserver keyserver.ubuntu.com --recv-key 467B942D3A79BD29
        gpg --export -a 467B942D3A79BD29 > mysql-repo.key    
        rpm --import mysql-repo.key
        
        {
        echo "[mysql-repo]"
        echo "name=MySQL ${DATABASE_VERSION} Community Server"
        echo "baseurl=https://repo.mysql.com/yum/mysql-${DATABASE_VERSION}-community/el/${DIST_VERSION}/\$basearch/"
        echo "gpgcheck=1"
        } > /etc/yum.repos.d/mysql.repo
    fi

    installPackages "mysql-community-server"

elif [[ "${DATABASE_TYPE}" == "mariadb" ]]; then
    echo "Installing mariadb-server packages..."

    echo "Using the setup script that is provided by mariadb..."
    wget https://downloads.mariadb.com/MariaDB/mariadb_repo_setup -O ${SCRIPT_DIR}/mariadb_repo_setup.sh
    chmod +x ${SCRIPT_DIR}/mariadb_repo_setup.sh

    if [[ true == "${ANY_DEB}" ]]; then
        DB_SERVICE_NAME=mysql
        DB_PACKAGE_NAME=mariadb-server
        # Adding mariadb key
        gpg --keyserver keyserver.ubuntu.com --recv-key 0xF1656F24C74CD1D8
        gpg --export -a 0xF1656F24C74CD1D8 > mariadb-repo.key
        apt-key add mariadb-repo.key
    elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
        if [[ "${DIST_VERSION}" == "6" ]]; then
            DB_SERVICE_NAME=mysql
            DB_PACKAGE_NAME=MariaDB-server
        else
            DB_SERVICE_NAME=mariadb
            DB_PACKAGE_NAME=mariadb-server
        fi
        
        gpg --keyserver keyserver.ubuntu.com --recv-key 5072E1F5
        gpg --export -a 5072E1F5 > mysql-repo.key    
        rpm --import mysql-repo.key
        
    fi

    echo "Executing prep script ${SCRIPT_DIR}/mariadb_repo_setup.sh ..."
    sleep 5

    ${SCRIPT_DIR}/mariadb_repo_setup.sh --skip-maxscale --mariadb-server-version=mariadb-${DATABASE_VERSION}

    installPackagesRetry ${DB_PACKAGE_NAME}
fi

if [[ "${OX_INIT}" == true ]]; then

    if [[ true == "${ANY_DEB}" ]]; then
        cp ${SCRIPT_DIR}/../config/database/ox-tunings.cnf /etc/mysql/conf.d
    elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
        cp ${SCRIPT_DIR}/../config/database/ox-tunings.cnf /etc/my.cnf.d
    fi

    restartService ${DB_SERVICE_NAME}
    waitPort localhost 3306

    if [[ -n "${DATABASE_ROOT_PASS}" ]]; then

        if [[ true == "${ANY_DEB}" ]]; then
            if [[ "${DATABASE_TYPE}" == "mysql" ]]; then
                echo "Setting MySQL root password..."
                /usr/bin/mysqladmin -u ${DATABASE_ROOT_USER} password ${DATABASE_ROOT_PASS}
                
                if [ "$DATABASE_ROOT_ALLOW_FROM_ALL" == true ] ; then
                    echo "Warning: Configuring mysql server to allow root connections from everywhere ..."
                    mysql -u ${DATABASE_ROOT_USER} -p${DATABASE_ROOT_PASS} -e "GRANT ALL PRIVILEGES ON *.* TO '${DATABASE_ROOT_USER}'@'%' IDENTIFIED BY '${DATABASE_ROOT_PASS}';"
                    mysql -u ${DATABASE_ROOT_USER} -p${DATABASE_ROOT_PASS} -e -D mysql "UPDATE mysql.user SET Grant_priv='Y', Super_priv='Y' WHERE User='${DATABASE_ROOT_USER}';FLUSH PRIVILEGES;" 
                    sed -i "s/.*bind-address.*/bind-address = 0.0.0.0/" /etc/mysql/mysql.conf.d/mysqld.cnf  

                    restartService ${DB_SERVICE_NAME}
                    waitPort localhost 3306     
                fi
            else
                # Nothing to do for mariadb on debian since it uses env vars?
                echo ""
            fi
        elif [[ true == "${ANY_RHEL}" ]] || [[ true == "${ANY_CENTOS}" ]]; then
            if [[ "${DATABASE_TYPE}" == "mysql" ]]; then
                if [[ ${DATABASE_VERSION} == "5.6" ]] ; then
                    echo "Setting MySQL root password..."
                    /usr/bin/mysqladmin -u ${DATABASE_ROOT_USER} password ${DATABASE_ROOT_PASS}
                elif [[ ${DATABASE_VERSION} == "5.7" ]]; then
                    # CentOS7 / RHEL(maybe) are special and generate a temporary root password for mysql
                    # We will grep it out and replace it with the provided one (if any)
                    password_match=`awk '/A temporary password is generated for/ {a=$0} END{ print a }' /var/log/mysqld.log | awk '{print $(NF)}'`
                    mysql -u ${DATABASE_ROOT_USER} -p$password_match --connect-expired-password -e "SET password FOR ${DATABASE_ROOT_USER}@localhost=password('$password_match');"
                    mysql -u ${DATABASE_ROOT_USER} -p$password_match -e "SHOW VARIABLES LIKE 'validate_password%';"
                    mysql -u ${DATABASE_ROOT_USER} -p$password_match -e "SET GLOBAL validate_password_length=4;"
                    mysql -u ${DATABASE_ROOT_USER} -p$password_match -e "SET GLOBAL validate_password_policy=LOW;"
                    mysql -u ${DATABASE_ROOT_USER} -p$password_match -e "SHOW VARIABLES LIKE 'validate_password%';"
                    mysql -u ${DATABASE_ROOT_USER} -p$password_match -e "SET password for ${DATABASE_ROOT_USER}@localhost=password('${DATABASE_ROOT_PASS}');"
                    mysql -u ${DATABASE_ROOT_USER} -p${DATABASE_ROOT_PASS} -e "flush privileges;"
                fi
                
                if [ "$DATABASE_ROOT_ALLOW_FROM_ALL" == true ] ; then
                    echo "Warning: Configuring mysql server to allow root connections from everywhere ..."
                    mysql -u ${DATABASE_ROOT_USER} -p${DATABASE_ROOT_PASS} -e "GRANT ALL PRIVILEGES ON *.* TO '${DATABASE_ROOT_USER}'@'%' IDENTIFIED BY '${DATABASE_ROOT_PASS}';"
                    mysql -u ${DATABASE_ROOT_USER} -p${DATABASE_ROOT_PASS} -e -D mysql "UPDATE mysql.user SET Grant_priv='Y', Super_priv='Y' WHERE User='${DATABASE_ROOT_USER}';FLUSH PRIVILEGES;" 
                    sed -i "s/.*bind-address.*/bind-address = 0.0.0.0/" /etc/mysql/mysql.conf.d/mysqld.cnf       

                    restartService ${DB_SERVICE_NAME}
                    waitPort localhost 3306     
                fi
            else
                # Nothing to do for mariadb on debian since it uses env vars?
                echo "Setting MySQL root password..."
                /usr/bin/mysqladmin -u root password ${DATABASE_ROOT_PASS}
            fi
        else
            echo "WARNING: What to do now? -> ${DIST} ${DIST_VERSION} "
            sleep 5
        fi

    else
        echo "WARNING: No root password provided, skipping password step ..."
    fi
fi


echo "Database setup done ..."